//
// GameManager.h
//

#ifndef __GAME_MANAGER_H__
#define __GAME_MANAGER_H__

#include <time.h>

#define DRAGONFLY_VERSION 1.0

#define DEFAULT_FRAME_TIME 33	// in milliseconds (33 ms == 30 fps)

class GameManager {

 private:
  GameManager (GameManager const&);	    // don't allow copy
  void operator=(GameManager const&);	    // don't allow assignment
  GameManager();			    // private since a singleton
  bool game_over;			    // true -> game loop should stop

 public:
  ~GameManager();

  // get the singleton instance of the GameManager
  static GameManager &getInstance();

  // start up all the GameManager services
  // append = true if add to log file (default false)
  // flush = true if flush after each write (default false)
  // seed is optional random seed (default is seed with system time)
  int startUp();
  int startUp(bool append, bool flush);
  int startUp(bool append, bool flush, time_t seed);

  // shut down the GameManager services, also terminating the program
  void shutDown();

  // run the game loop
  // frame_time is time between frames (default FRAME_TIME, indicated above)
  void run();
  void run(int frame_time);

  // indicate the game is over, which will stop the game loop
  void setGameOver();
};

#endif // __GAME_MANAGER_H__
