//
// WorldManager.h
//

#ifndef __GAME_WORLD_H__
#define __GAME_WORLD_H__

#include "EventObserver.h"
#include "GameObjectList.h"
#include "Position.h"
#include "Box.h"

class WorldManager : public EventObserver {

 protected:
  int obj_id;			       // next object id to assign
  WorldManager (WorldManager const&);  // don't allow copy
  void operator=(WorldManager const&); // don't allow assignment
  WorldManager();		       // private since a singleton
  GameObjectList obj;		       // list of all game objects
  GameObjectList del;		       // list of all game objects to delete
  Box boundary;			       // world boundaries
  Box view;			       // player window view
  bool valueInRange(int value,	       // true if min <= value <= max
		    int min, int max);

 public:
  ~WorldManager();
  static WorldManager &getInstance();

  // indicated object is to be deleted at end of current game loop
  int markForDelete(GameObject *p_go);

  // returns a list of all objects (obj)
  GameObjectList getAllObjects(void);

  // add object to game world (obj)
  int insertObj(GameObject *p_go);

  // remove object to game world (obj)
  int removeObj(GameObject *p_go);
  GameObject *findObj(int id);
  void update();
  void draw();
  int moveObj(GameObject *p_go, Position where);
  bool boxesIntersect(Box box1, Box box2);
  Box getBoundary();
  void setBoundary(Box new_boundary);
  Box getView();
  void setView(Box new_view);

  // check collision with any other solid game object
  // return pointer to first object collided with, else return NULL
  GameObject *isCollision(GameObject *p_go, Position where);
};

#endif // __GAME_WORLD_H__
