//
// Bullet.cpp
//

#include "Bullet.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "Saucer.h"

Bullet::Bullet(Position hero_pos) {

#ifdef DEBUG
  LogManager &log_manager = LogManager::getInstance();
  log_manager.writeLog("Bullet::Bullet(): Just added to game world");
#endif

  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  // link to "bullet" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  Sprite *p_temp_sprite = rm.getSprite("bullet");
  if (!p_temp_sprite) {
    LogManager &lm = LogManager::getInstance();
    lm.writeLog("Bullet::Bullet(): Warning! Sprite '%s' not found", "bullet");
    return;
  }
  setSprite(p_temp_sprite);
  setSpriteSlowdown(5);		

  // set other object properties
  setType("Bullet");
  move_slowdown = 1;
  move_countdown = move_slowdown;

  // set starting location, based on the hero's position passed in
  pos.setX(hero_pos.getX()+3);
  pos.setY(hero_pos.getY());
}

Bullet::~Bullet() {
#ifdef DEBUG
  LogManager &log_manager = LogManager::getInstance();
  log_manager.writeLog("Bullet::~Bullet(): In destructor");
#endif
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle event
// return 0 if ignored, else 1
int Bullet::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) {
    out();
    return 1;
  }

  if (e->getType() == COLLISION_EVENT) {
    EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    hit(p_collision_event);
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

void Bullet::move() {
  move_countdown--;
  if (move_countdown > 0)
    return;
  move_countdown = move_slowdown;
  Position new_pos(pos.getX() + 1, pos.getY());
  WorldManager &game_world = WorldManager::getInstance();
  game_world.moveObj(this, new_pos);
  return;
}

// if bullet moves outside world, destroy itself
void Bullet::out() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.markForDelete(this);
}

// if bullet hits saucer, mark saucer and bullet for deletion
void Bullet::hit(EventCollision *p_c) {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.markForDelete(p_c->getObj1());
  world_manager.markForDelete(p_c->getObj2());
}


