//
// Saucer.cpp
//

#include <stdlib.h>		// for random()

#include "Saucer.h"
#include "Hero.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "EventCollision.h"
#include "EventNuke.h"
#include "Explosion.h"
#include "Alien.h"

Saucer::Saucer() {
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  ResourceManager &resource_manager = ResourceManager::getInstance();

  // register interest in STEP event, generated each frame
  world_manager.registerInterest(this, STEP_EVENT);
  world_manager.registerInterest(this, NUKE_EVENT);

  // setup "saucer" sprite
  Sprite *p_temp_sprite = resource_manager.getSprite("saucer");
  if (!p_temp_sprite) {
    log_manager.writeLog("Saucer::Saucer(): Warning! Sprite '%s' not found", 
			 "saucer");
    return;
  }
  setSprite(p_temp_sprite, true); // center and define bounding box
  setSpriteSlowdown(4);		

  // set other object properties
  setType("Saucer");
  move_slowdown = 4;
  move_countdown = move_slowdown;

  // set starting location in the middle of the screen
  moveToStart();
}

Saucer::~Saucer() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
  world_manager.unregisterInterest(this, NUKE_EVENT);

  // create an explosion
  Explosion *p_explosion = new Explosion;
  p_explosion -> setPos(this -> getPos());

  // throw an alien
  Alien *p_alien = new Alien;
  p_alien -> setPos(this -> getPos());
  Position new_pos(pos.getX()-4, pos.getY());
  world_manager.moveObj(p_alien, new_pos);

  // saucers stay around perpetually
  new Saucer;
}

// handle event
// return 0 if ignored, else 1
int Saucer::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) {
    out();
    return 1;
  }

  if (e->getType() == COLLISION_EVENT) {
    EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    hit(p_collision_event);
    return 1;
  }

  if (e->getType() == NUKE_EVENT) {
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.markForDelete(this);
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

void Saucer::move() {
  move_countdown--;
  if (move_countdown > 0)
    return;
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.moveObj(this, new_pos);

  return;
}

// if moved off left edge, move back to far right
void Saucer::out() {

  // if we haven't moved off left edge, then nothing to do
  if (getPos().getX() >= 0)
    return;

  // otherwise, move back to far right
  moveToStart();

  // create a new Saucer
  new Saucer;
}

// move saucer to starting location on right side of screen
void Saucer::moveToStart() {
  WorldManager &world_manager = WorldManager::getInstance();
  Position temp_pos;

  int world_horiz = world_manager.getBoundary().getHorizontal();
  int world_vert = world_manager.getBoundary().getVertical();

  // x is off the right side of screen
  temp_pos.setX(world_horiz + random()%world_horiz + 3);

  // y is in the vertical range
  temp_pos.setY(random()%(world_vert-1) + 1);

  // if collision, move right slightly until empty space
  while (world_manager.isCollision(this, temp_pos))
    temp_pos.setX(temp_pos.getX()+1);

  world_manager.moveObj(this, temp_pos);

}

// if saucer and player collide, delete both
void Saucer::hit(EventCollision *p_c) {
  WorldManager &wm = WorldManager::getInstance();

  // if hero, mark this saucer for deletion
  //   and alert hero to collision
  if((p_c -> getObj1() -> getType()) == "Hero") {
    Hero *p_temp_hero = (Hero*)(p_c -> getObj1());
    p_temp_hero -> hit();
    wm.markForDelete(this);
  }

  if((p_c -> getObj2() -> getType()) == "Hero") {
    Hero *p_temp_hero = (Hero*)(p_c -> getObj2());
    p_temp_hero -> hit();
    wm.markForDelete(this);
  }
}
