//
// Star.cpp
//

#include "Star.h"
#include <stdlib.h>
#include "WorldManager.h"
#include "GraphicsManager.h"
#include "EventStep.h"
#include "EventOut.h"

Star::Star() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  setType("Star");
  is_solid = false;

  move_slowdown = random()%10;
  move_countdown = move_slowdown;

  altitude = -1;   // make them in the background

  pos.setX(random()%world_manager.getBoundary().getHorizontal());
  pos.setY(random()%world_manager.getBoundary().getVertical());
}

Star::~Star() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

void Star::draw() {
  GraphicsManager &graph_mgr = GraphicsManager::getInstance();
  graph_mgr.drawCh(pos, STAR_CHAR);
}

int Star::eventHandler(Event *e) {
  if( e->getType() == STEP_EVENT)
    {
      move();
      return 1;
    }

  if( e->getType() == OUT_EVENT)
    {
      out();
      return 1;
    }

  return 0;
}

void Star::move() {
  move_countdown--;
  if(move_countdown > 0)
    {
      return;
    }
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.moveObj(this, new_pos);
}

// if star moved off screen, move back to far right
void Star::out() {
  WorldManager &world_manager = WorldManager::getInstance();
  pos.setX(world_manager.getBoundary().getHorizontal() + random()%20);
  pos.setY(random()%world_manager.getBoundary().getVertical());
  move_slowdown = random()%10;
}
